"""
    Monitor

    ...
"""
import logging as log
import re
from pathlib import Path

from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer


class Handler(FileSystemEventHandler):
    """Monitor workflow for creation of 'cmd*.output files"""

    def __init__(self, cmdlogs, match, log_level):
        self.cmdlogs = cmdlogs
        self.match = match
        self.log_level = log_level

    def on_created(self, event):
        path = Path(event.src_path).resolve()
        if not re.match(self.match, str(path.name)):
            return

        self.cmdlogs.append(path)
        if not self.log_level:
            return

        log.info(f"cmd.output: {path}")


class WorkflowMonitor(object):
    def __init__(self, path, log_level):
        self.path = path
        self.cmdlogs = []
        self.handler = Handler(self.cmdlogs, r"cmd_\d+\.output", log_level)
        self.observer = Observer()

    def latest_cmdlog(self):
        """Returns the last created 'cmd*.output' file; if any."""

        if not self.cmdlogs:
            return None

        return self.cmdlogs[-1]

    def start(self):
        self.observer.schedule(self.handler, self.path, recursive=True)
        self.observer.start()

    def stop(self):
        self.observer.stop()
        self.observer.join()



"""
testrunner
==========

Invokes 'pytest' for cijoe testcases

It is intended for a specific use of pytest with the plugins listed below, it is as
such not a completely open-ended pytest-invocation as the cijoe-testcases must utilize
'cijoe.run' etc. to encapsulate command-execution, capture command output, and
collector artifacts.

Requires the following pytest plugins for correct behaviour:

 * cijoe, fixtures providing 'cijoe' object and "--config" and "--output"
 pytest-arguments to instantiate cijoe.

 * report-log, dump testnode-status as JSON, this is consumed by 'core.report' to
 produce an overview of testcases and link them with the cijoe-captured output and
 auxiliary files.

Retargetable: False
-------------------

Although the script is not retargetable, then the pytest-plugin itself is retargetable.
"""


def main(args, cijoe, step):
    """Invoke test-runner"""

    pytest_cmd = ["pytest"]
    pytest_cmd += ["--output", str(args.output / cijoe.output_ident)]
    pytest_cmd += [
        "--report-log",
        str(args.output / cijoe.output_ident / "testreport.log"),
    ]

    if args.config:
        pytest_cmd.append("--config")
        pytest_cmd.append(str(args.config))

    pytest_cmd += step.get("with").get("args", "").split(" ")

    err, _ = cijoe.run_local(" ".join(pytest_cmd))

    return err
